@@ -8,6 +8,7 @@ from traitlets import Integer, default
 from traitlets.config import Configurable
 from functools import partial
 
+
 def execute_cmd(cmd, **kwargs):
     """
     Call given command, yielding output line by line
@@ -23,6 +24,7 @@ def execute_cmd(cmd, **kwargs):
     # This should behave the same as .readline(), but splits on `\r` OR `\n`,
     # not just `\n`.
     buf = []
+
     def flush():
         line = b''.join(buf).decode('utf8', 'replace')
         buf[:] = []
@@ -42,34 +44,97 @@ def execute_cmd(cmd, **kwargs):
         if ret != 0:
             raise subprocess.CalledProcessError(ret, cmd)
 
+
 class GitPuller(Configurable):
-    depth = Integer(None, allow_none=True, config=True,
-                    help="""
-                    Depth (ie, commit count) to which to perform a
-                    shallow git clone.
+    depth = Integer(
+        config=True,
+        help="""
+        Depth (ie, commit count) of clone operations. Set this to 0 to make a
+        full depth clone.
 
-                    If not set, disables shallow clones.
-
-                    Defaults to reading from the NBGITPULLER_DEPTH
-                    environment variable.
-                    """)
+        Defaults to the value of the environment variable NBGITPULLER_DEPTH, or
+        1 if the the environment variable isn't set.
+        """
+    )
 
     @default('depth')
     def _depth_default(self):
-        depth = os.environ.get('NBGITPULLER_DEPTH')
-        if depth:
-            return int(depth)
-        return None
+        """This is a workaround for setting the same default directly in the
+        definition of the traitlet above. Without it, the test fails because a
+        change in the environment variable has no impact. I think this is a
+        consequence of the tests not starting with a totally clean environment
+        where the GitPuller class hadn't been loaded already."""
+        return int(os.environ.get('NBGITPULLER_DEPTH', 1))
 
-    def __init__(self, git_url, branch_name, repo_dir, **kwargs):
-        assert git_url and branch_name
+    def __init__(self, git_url, repo_dir, **kwargs):
+        assert git_url
 
         self.git_url = git_url
-        self.branch_name = branch_name
+        self.branch_name = kwargs.pop("branch")
+
+        if self.branch_name is None:
+            self.branch_name = self.resolve_default_branch()
+        elif not self.branch_exists(self.branch_name):
+            raise ValueError(f"Branch: {self.branch_name} -- not found in repo: {self.git_url}")
+
         self.repo_dir = repo_dir
         newargs = {k: v for k, v in kwargs.items() if v is not None}
         super(GitPuller, self).__init__(**newargs)
 
+    def branch_exists(self, branch):
+        """
+        This checks to make sure the branch we are told to access
+        exists in the repo
+        """
+        try:
+            heads = subprocess.run(
+                ["git", "ls-remote", "--heads", "--", self.git_url],
+                capture_output=True,
+                text=True,
+                check=True
+            )
+            tags = subprocess.run(
+                ["git", "ls-remote", "--tags", "--", self.git_url],
+                capture_output=True,
+                text=True,
+                check=True
+            )
+            lines = heads.stdout.splitlines() + tags.stdout.splitlines()
+            branches = []
+            for line in lines:
+                _, ref = line.split()
+                refs, heads, branch_name = ref.split("/", 2)
+                branches.append(branch_name)
+            return branch in branches
+        except subprocess.CalledProcessError:
+            m = f"Problem accessing list of branches and/or tags: {self.git_url}"
+            logging.exception(m)
+            raise ValueError(m)
+
+    def resolve_default_branch(self):
+        """
+        This will resolve the default branch of the repo in
+        the case where the branch given does not exist
+        """
+        try:
+            head_branch = subprocess.run(
+                ["git", "ls-remote", "--symref", "--", self.git_url, "HEAD"],
+                capture_output=True,
+                text=True,
+                check=True
+            )
+            for line in head_branch.stdout.splitlines():
+                if line.startswith("ref:"):
+                    # line resembles --> ref: refs/heads/main HEAD
+                    _, ref, head = line.split()
+                    refs, heads, branch_name = ref.split("/", 2)
+                    return branch_name
+            raise ValueError(f"default branch not found in {self.git_url}")
+        except subprocess.CalledProcessError:
+            m = f"Problem accessing HEAD branch: {self.git_url}"
+            logging.exception(m)
+            raise ValueError(m)
+
     def pull(self):
         """
         Pull selected repo from a remote git repository,
@@ -89,11 +154,10 @@ class GitPuller(Configurable):
         if self.depth and self.depth > 0:
             clone_args.extend(['--depth', str(self.depth)])
         clone_args.extend(['--branch', self.branch_name])
-        clone_args.extend([self.git_url, self.repo_dir])
+        clone_args.extend(["--", self.git_url, self.repo_dir])
         yield from execute_cmd(clone_args)
         logging.info('Repo {} initialized'.format(self.repo_dir))
 
-
     def reset_deleted_files(self):
         """
         Runs the equivalent of git checkout -- <file> for each file that was
@@ -103,8 +167,8 @@ class GitPuller(Configurable):
 
         yield from self.ensure_lock()
         deleted_files = subprocess.check_output([
-            'git', 'ls-files', '--deleted'
-        ], cwd=self.repo_dir).decode().strip().split('\n')
+            'git', 'ls-files', '--deleted', '-z'
+        ], cwd=self.repo_dir).decode().strip().split('\0')
 
         for filename in deleted_files:
             if filename:  # Filter out empty lines
@@ -132,7 +196,7 @@ class GitPuller(Configurable):
         Return list of files that have been changed upstream belonging to a particular kind of change
         """
         output = subprocess.check_output([
-            'git', 'log', '{}..origin/{}'.format(self.branch_name, self.branch_name),
+            'git', 'log', '..origin/{}'.format(self.branch_name),
             '--oneline', '--name-status'
         ], cwd=self.repo_dir).decode()
         files = []
@@ -182,7 +246,6 @@ class GitPuller(Configurable):
                 os.rename(f, new_file_name)
                 yield 'Renamed {} to {} to avoid conflict with upstream'.format(f, new_file_name)
 
-
     def update(self):
         """
         Do the pulling if necessary
@@ -230,7 +293,6 @@ class GitPuller(Configurable):
         ], cwd=self.repo_dir)
 
 
-
 def main():
     """
     Synchronizes a github repository with a local repository.
@@ -241,13 +303,17 @@ def main():
 
     parser = argparse.ArgumentParser(description='Synchronizes a github repository with a local repository.')
     parser.add_argument('git_url', help='Url of the repo to sync')
-    parser.add_argument('branch_name', default='master', help='Branch of repo to sync', nargs='?')
+    parser.add_argument('branch_name', default=None, help='Branch of repo to sync', nargs='?')
     parser.add_argument('repo_dir', default='.', help='Path to clone repo under', nargs='?')
     args = parser.parse_args()
 
     for line in GitPuller(
         args.git_url,
-        args.branch_name,
-        args.repo_dir
+        args.repo_dir,
+        branch=args.branch_name if args.branch_name else None
     ).pull():
         print(line)
+
+
+if __name__ == '__main__':
+    main()
